@@ -1,9 +1,12 @@
 import sys
 import io
 
+from itertools import product
+from enum import Enum
 from copy import deepcopy
 from collections import namedtuple
 from contextlib import contextmanager
+from typing import Union
 
 try:
     import sqlite3
@@ -21,8 +24,9 @@ except ImportError:
     mysql = None
 
 
-__version__ = "0.5.1"
+__version__ = "0.6.2"
 __all__ = [
+    "Association",
     "Database",
     "Field",
     "fn",
@@ -275,6 +279,7 @@ class Field(metaclass=FieldMetaclass):
     def __init__(
         self, type, name=None, caster=None, null=True, default=None, primary_key=False
     ):
+        self.schema: Union[Schema, None] = None
         self.type = type
         self.name = name
 
@@ -303,7 +308,7 @@ class Field(metaclass=FieldMetaclass):
 
 class QF(Field):
     """
-    An extra user defined field used for queries.
+    Query Field - an extra user defined field used for queries.
     This is mainly needed for referencing aliases.
 
     :param name: The name of the field.
@@ -321,6 +326,34 @@ class QF(Field):
         return f"<QF name={self.name}>"
 
 
+_Association = namedtuple(
+    "_Association", ["cardinality", "name", "schema", "owner", "field"]
+)
+
+
+class _Cardinals(Enum):
+    ONE_TO_ONE = 1
+    ONE_TO_MANY = 2
+
+
+class Association:
+    """
+    One to One/Many associations to help translate between relational data and object
+    data.
+
+    Many to Many is currently not supported because it does not translate well into
+    an object structure.
+    """
+
+    @staticmethod
+    def has_one(schema, on=[]):
+        return _Association(_Cardinals.ONE_TO_ONE, None, schema, on[0], on[1])
+
+    @staticmethod
+    def has_many(schema, on=[]):
+        return _Association(_Cardinals.ONE_TO_MANY, None, schema, on[0], on[1])
+
+
 class SchemaMetaclass(type):
     def __new__(cls, clsname, bases, attrs):
         # Deepcopy inherited fields
@@ -346,6 +379,9 @@ class SchemaMetaclass(type):
                 if f.name is None:
                     f.name = key
 
+            if isinstance(f, _Association):
+                setattr(c, key, f._replace(name=key))
+
         return c
 
     @property
@@ -356,6 +392,14 @@ class SchemaMetaclass(type):
             if isinstance(getattr(cls, key), Field)
         ]
 
+    @property
+    def associations(cls):
+        return [
+            getattr(cls, key)
+            for key in dir(cls)
+            if isinstance(getattr(cls, key), _Association)
+        ]
+
     @property
     def pk(cls):
         pk = None
@@ -399,10 +443,12 @@ class Schema(metaclass=SchemaMetaclass):
                 # >>> row.get("field") == None  # Field is not set
                 value = row[field.name]
             except KeyError:
-                # We didn't pass this field in, so move on if we're updating
                 if updating is True:
                     continue
 
+                if field.default is None:
+                    continue
+
             # Apply a default if we are inserting
             if value is None and updating is False and field.default is not None:
                 value = field.default
@@ -501,7 +547,7 @@ class Schema(metaclass=SchemaMetaclass):
 
     @classmethod
     def delete_by_pk(cls, id, new):
-        return cls.delete({cls.pk.name: id}, new)
+        return cls.delete({cls.pk.name: id})
 
 
 class QueryMetaclass(type):
@@ -535,12 +581,52 @@ class QueryMetaclass(type):
 Node = namedtuple("Node", ["node", "params"])
 
 
+def _do_preload_query(db, cardinality, query, value):
+    if cardinality == _Cardinals.ONE_TO_ONE:
+        return db.get_or_none(query, (value,))
+    elif cardinality == _Cardinals.ONE_TO_MANY:
+        return db.select(query, (value,))
+
+
+def _do_preload(db, association, row):
+    if isinstance(association, _Association):
+        query = f"""
+            select * from {association.schema.__tablename__}
+            where {association.field} = %s
+        """
+
+        return association.name, _do_preload_query(
+            db, association.cardinality, query, row[association.owner]
+        )
+
+    aso, values = list(association.items())[0]
+    associations = [
+        v for v in values if isinstance(v, _Association) or isinstance(v, dict)
+    ]
+    fields = [v.name for v in values if isinstance(v, Field)]
+    select = ", ".join(fields) if len(fields) > 0 else "*"
+
+    query = f"""
+        select {select} from {aso.schema.__tablename__}
+        where {aso.field} = %s
+    """
+
+    new_row = _do_preload_query(db, aso.cardinality, query, row[aso.owner])
+
+    for field_aso in associations:
+        name, field = _do_preload(db, field_aso, new_row)
+        new_row[name] = field
+
+    return aso.name, new_row
+
+
 class Query(metaclass=QueryMetaclass):
     def __init__(self, schema):
         self.schema = schema
 
         self._method = None
         self._nodes = []
+        self._preloads = []
 
     def _add_node(self, node, params):
         self._nodes.append(Node(_strip(node), params))
@@ -671,9 +757,24 @@ class Query(metaclass=QueryMetaclass):
 
         return self
 
-    def execute(self):
+    def preload(self, association):
+        self._preloads.append(association)
+        return self
+
+    def execute(self) -> dict:
         func = getattr(self.schema._database_, self._method)
-        return func(self._query, self._params)
+        data = func(self._query, self._params)
+
+        if data is None:
+            return data
+
+        for association, row in product(
+            self._preloads, data if isinstance(data, list) else [data]
+        ):
+            key, new_row = _do_preload(self.schema._database_, association, row)
+            row[key] = new_row
+
+        return data
 
     def copy(self):
         return deepcopy(self)
@@ -707,7 +808,7 @@ def _get_connection(func):
 
         f = func(self, *args, **kwargs)
 
-        if self.autoconnect is True:
+        if self.autoconnect is True and self.is_trans is False:
             self.close()
 
         return f
@@ -770,6 +871,9 @@ class Database:
         finally:
             self.is_trans = False
 
+            if self.autoconnect is True:
+                self.close()
+
     @_replace_placeholders
     def _execute(self, query, params):
         self.cursor.execute(query, params)
